/*
 * Copyright (c) 1997 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to use, copy, modify, and distribute the Software without
 * restriction, provided the Software, including any modified copies made
 * under this license, is not distributed for a fee, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * Except as contained in this notice, the name of the Massachusetts
 * Institute of Technology shall not be used in advertising or otherwise
 * to promote the sale, use or other dealings in this Software without
 * prior written authorization from the Massachusetts Institute of
 * Technology.
 *  
 */

#ifndef FFTWND_SHMEM_H

#define FFTWND_SHMEM_H

#include <fftw.h>
#include <transpose_shmem.h>

typedef struct {
    fftwnd_plan p_fft;
    fftw_plan p_last, p_first;
    transpose_shmem_plan p_transpose, p_transpose_inv;
} fftwnd_shmem_aux_data;

typedef fftwnd_shmem_aux_data *fftwnd_shmem_plan;

typedef enum {
    FFTW_NORMAL_ORDER,
    FFTW_TRANSPOSED_ORDER
} fftwnd_shmem_output_order;

extern fftwnd_shmem_plan fftwnd_shmem_create_plan(int rank, const int *n,
						  fftw_direction dir,
						  int flags);
extern fftwnd_shmem_plan fftw2d_shmem_create_plan(int nx, int ny,
						  fftw_direction dir,
						  int flags);
extern fftwnd_shmem_plan fftw3d_shmem_create_plan(int nx, int ny, int nz,
						  fftw_direction dir,
						  int flags);

extern void fftwnd_shmem_destroy_plan(fftwnd_shmem_plan p);

extern void fftwnd_shmem_local_sizes(fftwnd_shmem_plan p,
				     int *local_nx,
				     int *local_x_start,
				     int *local_ny_after_transpose,
				     int *local_y_start_after_transpose,
				     int *total_local_size);

extern void fftwnd_shmem(fftwnd_shmem_plan p,
			 int n_fields,
			 FFTW_COMPLEX *local_data,
			 FFTW_COMPLEX *work,
			 fftwnd_shmem_output_order output_order);

#endif
