/*
 * Copyright (c) 1997 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to use, copy, modify, and distribute the Software without
 * restriction, provided the Software, including any modified copies made
 * under this license, is not distributed for a fee, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * Except as contained in this notice, the name of the Massachusetts
 * Institute of Technology shall not be used in advertising or otherwise
 * to promote the sale, use or other dealings in this Software without
 * prior written authorization from the Massachusetts Institute of
 * Technology.
 *  
 */

#ifndef TRANSPOSE_SHMEM_H

#define TRANSPOSE_SHMEM_H

typedef struct {
     int block_num, dest_pe, send_size, dest_offset;
} transpose_shmem_block_dest;

typedef struct {
     int output_in_place;

     int n_pes, my_pe;
     
     int nx,ny,local_nx,local_ny;

     transpose_shmem_block_dest *block_dest;
     int num_blocks, block_size;
     int block_size_transpose;

     int *perm_block_dest;
     int num_perm_blocks, perm_block_size;

     char *move;
     char move_size;
} transpose_shmem_plan_struct;

typedef transpose_shmem_plan_struct *transpose_shmem_plan;

extern void transpose_shmem_get_local_size(int n, int my_pe, int n_pes,
					 int *local_n, int *local_start);
extern int transpose_shmem_get_local_storage_size(int nx, int ny,
						int my_pe, int n_pes);

extern transpose_shmem_plan transpose_shmem_create_plan(int nx, int ny);
extern void transpose_shmem_destroy_plan(transpose_shmem_plan p);

extern void transpose_shmem(transpose_shmem_plan p, 
			    double *local_data, 
			    double *work,
			    int el_size);

#define TRANSPOSE_SHMEM_OUTPUT_IN_PLACE 0

#endif
